package com.telelogic.tau.tor;

public final class TopRegion extends Region
{
    // Event corresponding to a request to run the initial transition of a top region
    public static class RunInitialTransition extends InternalEvent 
    {
        public static boolean isTypeOf(Event e)
        { 
            return e instanceof RunInitialTransition; 
        }
    }

	
	public TopRegion(StateMachine sm)
	{
		super(null);
		m_sm = sm;		
	}
	
	@Override
	public Dispatchable.EventAction execute(Event e)
	{
	    // Handle RunInitialTransition event
	    if (RunInitialTransition.isTypeOf(e)) {
	        initialTransition();	        
	        return Dispatchable.EventAction.Consumed;
	    }

	    return super.execute(e);
	}
	
	@Override
	public void initialTransition() 
	{
		if (m_sm != null)
			m_sm.initialTransition();
	}

	@Override
	public StateMachine getStateMachine()
	{
		return m_sm;
	}

    // Overloaded to call the initialTransition function asynchronously.
	@Override
	public void callInitialTransition() 
	{
		// Special case for the TopRegion of the top statemachine of a dispatchable class:
	    // Post a request for calling the initialTransition rather than doing it immediately.
	    // This is because the start transition should be executed in the context of the 
	    // dispatcher rather than in the context of the caller of 'start'.
	    StateMachine sm = getStateMachine();
	    if (sm != null) {
	        DispatchableClass dc = sm.getDispatchableClass();
	        if (dc != null && (dc.getClassifierBehavior() == sm)) {
	            Utilities.sendTo(new RunInitialTransition(), dc);
	            return;
	        }
	    }

	    // For all other cases, fallback to the inherited implementation.
	    super.callInitialTransition();
	}
	
	@Override
	public void finish(ExitPoint ep)
	{
		if (ep != null && ep.m_owner == this) 
		{
			// If a valid exit point was passed, leave the owning statemachine 
			// through that exit point.
			StateMachine sm = getStateMachine();
			if (sm != null)
				sm.leave(ep);
		} 
		else 
		{
			// Otherwise, mark this region as finished, and if all top level
			// regions are finished, finish the statemachine.
			m_finished = true;
			StateMachine sm = getStateMachine();
			if (sm != null && !sm.isSubStateMachine() && sm.isFinished()) 	    
				sm.finish();	    
		}  
	}

	
	// Overridden to call exitPointTransitions in the state machine.
	@Override
	public void exitPointTransitions(ExitPoint ep) 
	{
		StateMachine sm = getStateMachine();
		if (sm != null)
			sm.exitPointTransitions(ep);
	}

	// Overridden to call entryPointTransitions in the state machine.
	@Override
	public void entryPointTransitions(EntryPoint ep) 
	{
		StateMachine sm = getStateMachine();
		if (sm != null)
			sm.entryPointTransitions(ep);
	}
	
	// The statemachine to which this region belongs.
	protected StateMachine m_sm;
}
